/****************************************************
 * Vilmos Kintera (DAXRunBase) - 2016/08/16 - V2.4  *
 * https://www.daxrunbase.com                       *
 * Replace [master] DB for table parsing function   *
 * Replace e-mail address and subject               *
 * Replace MyAXDB with your AX database             *
 ****************************************************/

USE [master]
GO

-- Declare variables
DECLARE @DatabaseMailProfile AS VARCHAR(MAX)	= 'MYEXCHANGE';
DECLARE @MailAddress AS VARCHAR(MAX)			= 'myitteam@alert.me';
DECLARE @MailSubject AS VARCHAR(MAX)			= 'AX Prod SysTraceTableSQL increment';

DECLARE @fromtime	AS datetime;
DECLARE @totime		AS datetime;

-- Date range is for today
SET @fromtime	= Convert(DateTime, DATEDIFF(DAY, 0, SYSDATETIME()))
SET @totime		= SYSDATETIME()

-- Create the Trace tracking table for last execution
IF NOT EXISTS(SELECT id FROM [dbo].[sysobjects] WHERE [name] = 'WIK_SysTraceTableSQL')
BEGIN
	-- Get the initial values
	CREATE TABLE [dbo].[WIK_SysTraceTableSQL] (
		Statement nvarchar(max) NULL,
		Hits int NOT NULL,
	)

	INSERT INTO [dbo].[WIK_SysTraceTableSQL]
	SELECT Statement, count(*) Hits FROM [MyAXDB].[dbo].[SysTraceTableSQL]
		WHERE CreatedDateTime >= @fromtime AND CreatedDateTime <= @totime
		GROUP BY Statement
		ORDER BY Hits DESC
END

-- Create the table parser function
IF OBJECTPROPERTY(OBJECT_ID('dbo.WIK_ParseStatement'), 'isTableFunction') IS NULL
BEGIN
	DECLARE @PlaceHolder VARCHAR(MAX)
	SET @PlaceHolder = 
		'/****** Object:  UserDefinedFunction [dbo].[WIK_ParseStatement]    Script Date: 16/06/2015 16:41:30 ******/
-- =============================================
-- Author:		<Vilmos Kintera>
-- Create date: <2015-06-15>
-- Description:	<Prase SQL statement for table names>
-- =============================================
CREATE FUNCTION [dbo].[WIK_ParseStatement]
(	
		@StringList NVARCHAR(MAX)
)
RETURNS @TableList TABLE(ID int identity(1,1) PRIMARY KEY, 
							[Substring] NVARCHAR(max))
BEGIN
	IF @StringList = '''' RETURN
	
	SET @StringList = replace(@StringList, ''<'', '' '')
	SET @StringList = replace(@StringList, ''>'', '' '')
	SET @StringList = replace(@StringList, ''?'', '' '')
	SET @StringList = replace(@StringList, '''''''', '' '')
	SET @StringList = replace(@StringList, ''\'', '' '')
	SET @StringList = replace(@StringList, ''&'', '' '')
	SET @StringList = replace(@StringList, ''('', '' '')
	SET @StringList = replace(@StringList, '')'', '' '')
	SET @StringList = replace(@StringList, ''	'', '' '')
	SET @StringList = replace(@StringList, ''  '', '' '')

	DECLARE @XML xml
	SET @XML = ''<root><csv>''+replace(@StringList, '' '', ''</csv><csv>'') + ''</csv></root>''

	DECLARE xml_cursor CURSOR FOR 
		SELECT rtrim(ltrim(replace(Word.value(''.'',''nvarchar(max)''),char(10),''''))) AS ListMember
			FROM @XML.nodes(''/root/csv'') AS WordList(Word);

	DECLARE @ListMember AS nvarchar(max)

	OPEN xml_cursor;
	FETCH NEXT FROM xml_cursor into @ListMember;

	DECLARE @IsTable AS binary = 0

	WHILE (@@FETCH_STATUS = 0)
	BEGIN
		IF (@IsTable = 1
			AND @ListMember NOT IN (''INTO'', ''T1''))
		BEGIN
			SET @IsTable = 0
			INSERT @TableList SELECT @ListMember
		END
		IF (@ListMember IN (''FROM'', ''JOIN'', ''EXECUTE'', ''INSERT'', ''UPDATE'', ''INTO''))
		BEGIN
			SET @IsTable = 1
		END
		FETCH NEXT FROM xml_cursor into @ListMember;
	END

	CLOSE xml_cursor;
	DEALLOCATE xml_cursor;

RETURN
END -- FUNCTION'
	EXEC(@PlaceHolder)
END


-- Get the current stats and the percent change since the initial capture
SELECT 
	ISNULL(OLD.Hits, 0) AS OldHits,
	NEW.Hits,
	NEW.Hits - ISNULL(OLD.Hits, 0) AS Increment,
	NEW.Statement
INTO #tmpStats FROM (
	SELECT Statement, count(*) Hits FROM [MyAXDB].[dbo].[SysTraceTableSQL]
		WHERE CreatedDateTime >= @fromtime AND CreatedDateTime <= @totime
		GROUP BY Statement
	) NEW
	LEFT OUTER JOIN (
		SELECT Statement, Hits FROM [dbo].[WIK_SysTraceTableSQL]
	) OLD
	ON NEW.Statement = OLD.Statement
	ORDER BY Hits DESC

DECLARE @totalchange DECIMAL(10, 2)
SELECT @totalchange = SUM(Increment) FROM #tmpStats

DECLARE @mailbody VARCHAR(max)
DECLARE @mailbodyLIKE VARCHAR(max)
DECLARE @mailheader VARCHAR(max)
DECLARE @mailfooter VARCHAR(max)

SET @mailheader = 
(
	SELECT '<html><head>Normal statements</head><br/><br/>'
)

SET @mailfooter = 
(
	SELECT '</html>'
)

SET @mailbody = 
(
	SELECT td = CAST(OldHits AS NVARCHAR(MAX)), '',
		td = CAST(Hits AS NVARCHAR(MAX)), '',
		td = CAST(Increment AS NVARCHAR(MAX)), '',
		td = (SELECT DISTINCT Substring + CHAR(10) FROM [dbo].[WIK_ParseStatement](#tmpStats.STATEMENT) FOR XML PATH('')), '',
		td = ISNULL(CAST(Statement AS NVARCHAR(MAX)), ' '), '',
		td = CASE WHEN @TotalChange > 0
			THEN CONVERT(DECIMAL(10,2), CONVERT(DECIMAL(10,2),Increment) / @totalchange * 100)
			ELSE 0
		END, ''
		FROM #tmpStats
		WHERE Increment > 0 AND STATEMENT IS NOT NULL AND STATEMENT NOT LIKE '% LIKE %'
	FOR XML PATH( 'tr' )
)

SET @mailbodyLIKE = 
(
	SELECT td = CAST(OldHits AS NVARCHAR(MAX)), '',
		td = CAST(Hits AS NVARCHAR(MAX)), '',
		td = CAST(Increment AS NVARCHAR(MAX)), '',
		td = (SELECT DISTINCT Substring + CHAR(10) FROM [dbo].[WIK_ParseStatement](#tmpStats.STATEMENT) FOR XML PATH('')), '',
		td = ISNULL(CAST(Statement AS NVARCHAR(MAX)), ' '), '',
		td = CASE WHEN @TotalChange > 0
			THEN CONVERT(DECIMAL(10,2), CONVERT(DECIMAL(10,2),Increment) / @totalchange * 100)
			ELSE 0
		END, ''
		FROM #tmpStats
		WHERE Increment > 0 AND STATEMENT IS NOT NULL AND STATEMENT LIKE '% LIKE %'
	FOR XML PATH( 'tr' )
)

SET @mailbody =
(
	SELECT '<body><table cellpadding="2" cellspacing="2" border="1" width="100%">'
	+ '<col style="width:10%">'
	+ '<col style="width:10%">'
	+ '<col style="width:10%">'
	+ '<col style="width:15%">'
	+ '<col style="width:45%">'
	+ '<col style="width:10%">'
    + '<thead><tr><th>Previous hits</th><th>Current hits</th><th>Increment</th><th>Tables</th><th>Statement</th><th>Percent</th></tr></thead>'    
    + '<tbody>' + ISNULL(@mailbody, '') + '</tbody>'
    + '</table></body>'
	-- LIKE statements section
	+ '<br/><br/><head>LIKE statements</head><br/><br/>'
	+ '<body><table cellpadding="2" cellspacing="2" border="1" width="100%">'
	+ '<col style="width:10%">'
	+ '<col style="width:10%">'
	+ '<col style="width:10%">'
	+ '<col style="width:15%">'
	+ '<col style="width:45%">'
	+ '<col style="width:10%">'
    + '<thead><tr><th>Previous hits</th><th>Current hits</th><th>Increment</th><th>Tables</th><th>Statement</th><th>Percent</th></tr></thead>'    
    + '<tbody>' + ISNULL(@mailbodyLIKE, '') + '</tbody>'
    + '</table></body>'
)

SET @mailbody =
(
	SELECT @mailheader + ISNULL(@mailbody, '') + @mailfooter
)

-- Send mail
IF EXISTS(SELECT TOP 1 * FROM #tmpStats WHERE Increment > 0 AND STATEMENT IS NOT NULL)
BEGIN
	EXEC msdb.dbo.sp_send_dbmail
		@recipients		= @MailAddress,
		@subject		= @MailSubject,
		@profile_name	= @DatabaseMailProfile,
		@body_format	= 'HTML',
		@body			= @mailbody;

	-- Repopulate table after execution
	TRUNCATE TABLE [dbo].[WIK_SysTraceTableSQL]

	INSERT INTO [dbo].[WIK_SysTraceTableSQL]
	SELECT Statement, count(*) Hits FROM [MyAXDB].[dbo].[SysTraceTableSQL]
		WHERE CreatedDateTime >= @fromtime AND CreatedDateTime <= @totime
		GROUP BY Statement
		ORDER BY Hits DESC
END

SELECT * FROM #tmpStats
WHERE STATEMENT IS NOT NULL
--AND Increment > 0
ORDER BY Increment DESC, Hits DESC

DROP TABLE #tmpStats
